@@ -1,7 +1,8 @@
 from .asgi_server import ASGIServer as ASGIServer
 from .websocket_provider import WebsocketProvider as WebsocketProvider
 from .websocket_server import WebsocketServer as WebsocketServer
-from .websocket_server import YRoom as YRoom
+from .websocket_server import exception_logger as exception_logger
+from .yroom import YRoom as YRoom
 from .yutils import YMessageType as YMessageType
 
-__version__ = "0.13.0"
+__version__ = "0.13.4"
