@@ -28,8 +28,10 @@ class BaseYStore(ABC):
     metadata_callback: Callable[[], Awaitable[bytes] | bytes] | None = None
     version = 2
     _started: Event | None = None
+    _stopped: Event | None = None
     _task_group: TaskGroup | None = None
-    __start_lock: Lock | None = None
+    _public_start_lock: Lock | None = None
+    _private_start_lock: Lock | None = None
 
     @abstractmethod
     def __init__(
@@ -50,11 +52,23 @@ class BaseYStore(ABC):
             self._started = Event()
         return self._started
 
+    @property
+    def stopped(self) -> Event:
+        if self._stopped is None:
+            self._stopped = Event()
+        return self._stopped
+
+    @property
+    def start_lock(self) -> Lock:
+        if self._public_start_lock is None:
+            self._public_start_lock = Lock()
+        return self._public_start_lock
+
     @property
     def _start_lock(self) -> Lock:
-        if self.__start_lock is None:
-            self.__start_lock = Lock()
-        return self.__start_lock
+        if self._private_start_lock is None:
+            self._private_start_lock = Lock()
+        return self._private_start_lock
 
     async def __aenter__(self) -> BaseYStore:
         async with self._start_lock:
@@ -96,12 +110,14 @@ class BaseYStore(ABC):
             async with create_task_group() as self._task_group:
                 task_status.started()
                 self.started.set()
+                await self.stopped.wait()
 
     async def stop(self) -> None:
         """Stop the store."""
         if self._task_group is None:
             raise RuntimeError("YStore not running")
 
+        self.stopped.set()
         self._task_group.cancel_scope.cancel()
         self._task_group = None
 
@@ -309,7 +325,7 @@ class SQLiteYStore(BaseYStore):
     document_ttl: int | None = None
     path: str
     lock: Lock
-    db_initialized: Event
+    db_initialized: Event | None
     _db: Connection
 
     def __init__(
@@ -329,7 +345,7 @@ class SQLiteYStore(BaseYStore):
         self.metadata_callback = metadata_callback
         self.log = log or getLogger(__name__)
         self.lock = Lock()
-        self.db_initialized = Event()
+        self.db_initialized = None
 
     async def start(
         self,
@@ -342,7 +358,6 @@ class SQLiteYStore(BaseYStore):
         Arguments:
             task_status: The status to set when the task has started.
         """
-
         self.db_initialized = Event()
         if from_context_manager:
             assert self._task_group is not None
@@ -358,10 +373,11 @@ class SQLiteYStore(BaseYStore):
                 self._task_group.start_soon(self._init_db)
                 task_status.started()
                 self.started.set()
+                await self.stopped.wait()
 
     async def stop(self) -> None:
         """Stop the store."""
-        if self.db_initialized.is_set():
+        if self.db_initialized is not None and self.db_initialized.is_set():
             await self._db.close()
         await super().stop()
 
@@ -407,6 +423,7 @@ class SQLiteYStore(BaseYStore):
                 await db.commit()
                 await db.close()
         self._db = await connect(self.db_path)
+        assert self.db_initialized is not None
         self.db_initialized.set()
 
     async def read(self) -> AsyncIterator[tuple[bytes, bytes, float]]:
@@ -415,6 +432,8 @@ class SQLiteYStore(BaseYStore):
         Returns:
             A tuple of (update, metadata, timestamp) for each update.
         """
+        if self.db_initialized is None:
+            raise RuntimeError("YStore not started")
         await self.db_initialized.wait()
         try:
             async with self.lock:
@@ -438,6 +457,8 @@ class SQLiteYStore(BaseYStore):
         Arguments:
             data: The update to store.
         """
+        if self.db_initialized is None:
+            raise RuntimeError("YStore not started")
         await self.db_initialized.wait()
         async with self.lock:
             # first, determine time elapsed since last update
